@@ -3,16 +3,17 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 sub new {
     return bless {}, shift;
 }
 sub mock {
-    
-    shift; # throw away class/object
+
+    shift; # throw away object/class
 
     my $self = bless {}, __PACKAGE__;
+
     my $sub = shift;
     %{ $self } = @_;
 
@@ -27,9 +28,13 @@ sub mock {
     }
 
     $self->_check_side_effect($self->{side_effect});
+    push @{ $self->{return} }, $self->{return_value};
 
     $self->{name} = $sub;
-    $self->{orig} = \&$sub if ! $fake;
+
+    if (! $fake) {
+        $self->{orig} = \&$sub;
+    }
 
     my $called;
 
@@ -41,10 +46,21 @@ sub mock {
             @{ $self->{called_with} } = @_;
             $self->{called_count} = ++$called;
             if ($self->{side_effect}) {
-                my $effect = $self->{side_effect}->();
-                return $effect if defined $effect;
+                if (wantarray){
+                    my @effect = $self->{side_effect}->(@_);
+                    return @effect if @effect;
+                }
+                else {
+                    my $effect = $self->{side_effect}->(@_);
+                    return $effect if defined $effect;
+                }
             }
-            return $self->{return_value} if defined $self->{return_value};
+
+            return undef if ! $self->{return};
+
+            return ! wantarray && @{ $self->{return} } == 1
+                ? $self->{return}[0]
+                : @{ $self->{return} };
         };
     }
     return $self;
@@ -59,7 +75,6 @@ sub unmock {
 
         if (defined $self->{orig}) {
             *$sub = \&{ $self->{orig} };
-            delete $self->{orig};
         }
         else {
             undef *$sub;
@@ -74,18 +89,24 @@ sub called_count {
     return shift->{called_count};
 }
 sub called_with {
-    return @{ shift->{called_with} };
+    my $self = shift;
+    if (! $self->called){
+        die "\n\ncan't call called_with() before the mocked sub has " .
+            "been called\n";
+    }
+    return @{ $self->{called_with} };
 }
 sub name {
     return shift->{name};  
 }
 sub reset {
-    for (qw(side_effect return_value called called_count called_with)){
+    for (qw(side_effect return_value return called called_count called_with)){
         delete $_[0]->{$_};
     }
 }
 sub return_value {
-    $_[0]->{return_value} = $_[1];
+    my $self = shift;
+    @{ $self->{return} } = @_;
 }
 sub side_effect {
     $_[0]->_check_side_effect($_[1]);
@@ -96,6 +117,12 @@ sub _check_side_effect {
         die "side_effect parameter must be a code reference";
     }
 }
+sub DESTROY {
+    my $self = shift;
+    if (defined $self->{orig} && ! $self->{keep_mock_on_destroy}) {
+        $self->unmock if !$self->{keep_mock_on_destroy};
+    }
+}
 sub _end {}; # vim fold placeholder
 
 1;
@@ -131,18 +158,28 @@ Mock::Sub - Mock module, package, object and standard subroutines, with unit tes
     my $foo = $mock->mock('Package::foo');
     my $bar = $mock->mock('Package::bar');
 
-    # have the mocked sub return something when it's called (you can use void
-    # context if you don't need the functionality of the object)
+    # have the mocked sub return something when it's called
+
+    # a single scalar only
+
+    my $foo = $mock->mock('Package::foo', return_value => 'True');
+
+    # or return a list:
+
+    $foo->return_value(1, 2, {a => 1});
 
-    $mock->mock('Package::foo', return_value => 'True');
     my $return = Package::foo;
 
     # have the mocked sub perform an action (void context again)
 
-    $mock->mock('Package::foo', side_effect => sub { die "eval catch"; });
+    my $foo = $mock->mock('Package::foo', side_effect => sub { die "eval catch"; });
     eval { Package::foo; };
     print 'died' if $@;
 
+    # add a side-effect after instantiation
+
+    $foo->side_effect(sub {print "hello, world!; });
+
     # extract the parameters the sub was called with (best if you know what
     # the original sub is expecting)
 
@@ -160,7 +197,8 @@ Mock::Sub - Mock module, package, object and standard subroutines, with unit tes
 
     $foo->reset;
 
-    # restore original functionality to the sub
+    # restore original functionality to the sub (we unmock() by default on
+    # DESTROY()
 
     $foo->unmock;
 
@@ -237,13 +275,17 @@ Instantiates and returns a new Mock::Sub object.
 Instantiates a new object on each call. 'sub' is the name of the subroutine
 to mock (requires full package name if the sub isn't in C<main::>).
 
+The mocked sub will return undef if a return value isn't set, or a side effect
+doesn't return anything.
+
 Options:
 
 =over 4
 
 =item C<return_value>
 
-Set this to have the mocked sub return anything you wish.
+Set this to have the mocked sub return anything you wish (accepts a single
+scalar only. See C<return_value()> method to return a list).
 
 =item C<side_effect>
 
@@ -259,6 +301,12 @@ To work around this and have the side_effect run but still get the
 return_value thereafter, write your cref to evaluate undef as the last thing
 it does: C<sub {...; undef; }>.
 
+=item C<keep_mock_on_destroy>
+
+By default, we restore original sub functionality after the mock object goes
+out of scope. You can keep the mocked sub in place by setting this parameter
+to any true value.
+
 =back
 
 =head2 C<unmock>
@@ -276,7 +324,8 @@ Returns the number of times the mocked sub has been called.
 
 =head2 C<called_with>
 
-Returns an array of the parameters sent to the subroutine.
+Returns an array of the parameters sent to the subroutine. C<dies()> if we're
+called before the mocked sub has been called.
 
 =head2 C<name>
 
@@ -290,8 +339,7 @@ they do for the C<side_effect> parameter.
 =head2 C<return_value>
 
 Add (or change, delete) the mocked sub's return value after instantiation.
-Takes a single parameter, which can be anything you want it to be. Send in
-C<undef> to remove a previously set value.
+Can be a scalar or list. Send in C<undef> to remove a previously set value.
 
 =head2 C<reset>
 
@@ -303,6 +351,10 @@ with C<called()> and C<called_count()> back to undef/false.
 I didn't make this a C<Test::> module (although it started that way) because
 I can see more uses than placing it into that category.
 
+Do not use a C<new> mock object to call C<mock()> in void context. There will
+be no object returned, thus no way to manually C<unmock()> the sub, nor any
+way for automated cleanup to revert it back either.
+
 =head1 AUTHOR
 
 Steve Bertrand, C<< <steveb at cpan.org> >>
